# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import division
import sys
import importlib

from django.conf import settings
from django.db import models
from django.contrib.auth.models import User
from ox.django import fields

def str_to_class(str):
    if '.' in str:
        module_name = '.'.join(str.split('.')[:-1])
        class_name = str.split('.')[-1]
        m = importlib.import_module(module_name)
        return getattr(m, class_name)
    else:
        return reduce(getattr, str.split("."), sys.modules[__name__])

class LSDItem(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)
    accessed = models.DateTimeField(auto_now=True)

    sha1 = models.CharField(max_length=40, unique=True)
    info_hash = models.CharField(max_length=40, unique=True, null=True)

    data = fields.DictField(default={}, editable=False)

Item = str_to_class(getattr(settings, 'LSD_ITEM', 'LSDItem'))

class Request(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    item = models.ForeignKey(Item, related_name='lsd_requests', db_index=True)
    user = models.ForeignKey(User, related_name='lsd_requests', db_index=True)

    @classmethod
    def get(cls, user):
        return [f['sha1'] for f in Item.objects.filter(
            lsd_files__in=File.objects.filter(user=user),
            lsd_requests__in=cls.objects.exclude(user=user)
        ).values('sha1')]

class File(models.Model):

    item = models.ForeignKey(Item, related_name='lsd_files', db_index=True)
    user = models.ForeignKey(User, related_name='lsd_files', db_index=True)

    @classmethod
    def get(cls, user):
        return [f['sha1'] for f in Item.objects.filter(
            lsd_files__in=cls.objects.filter(user=user)
        ).values('sha1')]

