# -*- coding: utf-8 -*-
# vi:si:et:sw=4:sts=4:ts=4
from __future__ import division
import json

from ox.django.decorators import login_required_json
from ox.django.shortcuts import render_to_json_response, json_response

import models

from ox.django.api import actions

@login_required_json
def sync(request):
    '''
        sync({}) -> {ids: [id]}
        sync({add: [id], remove: [id]}) -> {update: [id]}
        sync({update: [metadata]}) -> {}
    '''
    response = json_response()
    if 'data' in request.POST:
        data = json.loads(request.POST['data'])
    else:
        data = {}
    if not data:
        response['data']['ids'] = models.File.get(request.user)
    update = []
    if 'add' in data:
        for sha1 in data['add']:
            i, created = models.Item.objects.get_or_create(sha1=sha1)
            if created:
                update.append(sha1)
            f, created = models.File.objects.get_or_create(item=i, user=request.user)
            models.Request.objects.filter(user=request.user, item=i).delete()
    if 'remove' in data:
        models.File.objects.filter(item__sha1__in=data['remove'], user=request.user).delete()
    if 'update' in data:
        for f in data['update']:
            i, created = models.Item.objects.get_or_create(sha1=f['sha1'])
            i.info_hash = f['info_hash']
            for key in f.keys():
                if key not in ('sha1', 'info_hash'):
                    i.data[key] = f[key]
            #i.update_external_data()
            i.save()
    response['data']['update'] = update
    return render_to_json_response(response)
actions.register(sync)

@login_required_json
def addRequests(request):
    '''
        addRequests({ids: [id]}) -> {metadata: [metadata]}
    '''
    response = json_response()
    data = json.loads(request.POST['data'])
    metadata = []
    for item in models.Item.objects.filter(sha1__in=data['ids']):
        info_hash = item.info_hash
        if info_hash and not request.user.lsd_files.filter(item=item).exists():
            r, created = models.Request.objects.get_or_create(user=request.user, item=item)
            metadata.append({'sha1': item.sha1, 'info_hash': info_hash})
    response['data']['metadata'] = metadata
    return render_to_json_response(response)
actions.register(addRequests)

@login_required_json
def removeRequests(request):
    '''
        removeRequests({ids: [id]}) -> {}
    '''
    response = json_response()
    data = json.loads(request.POST['data'])
    models.Request.objects.filter(user=request.user, item__sha1__in=data['ids']).delete()
    return render_to_json_response(response)
actions.register(removeRequests)

@login_required_json
def getRequests(request):
    '''
        getRequests({}) -> {ids: [id]}
    '''
    response = json_response()
    response['data']['ids'] = models.Request.get(request.user)
    return render_to_json_response(response)
actions.register(getRequests)
